
#include "framework.h"
#include "iRecHS2mrc.h"
#include "darknoise.h"
#define MAX_STRING_LENGTH 256

using namespace boost;
using namespace std;

HANDLE MRC::hStdOutput;

int MRC::a2t(char* src, TCHAR* des, rsize_t num)
{
	return MultiByteToWideChar(CP_OEMCP, MB_PRECOMPOSED, src, (int)num, des, (int)num);
}

int MRC::t2a(TCHAR* src, char* des, size_t num)
{
	return WideCharToMultiByte(CP_THREAD_ACP, 0, src, (int)num, des, (int)num, NULL, NULL);
}

CameraImage::CameraImage()
{
	w = h = 0;
	pixelbits = 0;
	TimeStamp = 0;
	FrameID = 0;
	CPU_FreqCount.QuadPart = 0;
	img = NULL;


	CaptureCounter = 0;
}

CameraImage::~CameraImage()
{
	if (img != NULL) delete[] img;


}





std::string MRC::wstr2str(const std::wstring& wstr)
{
	if (wstr.empty()) return std::string();
	int size_needed = WideCharToMultiByte(CP_UTF8, 0, &wstr[0], (int)wstr.size(), NULL, 0, NULL, NULL);
	std::string strTo(size_needed, 0);
	WideCharToMultiByte(CP_UTF8, 0, &wstr[0], (int)wstr.size(), &strTo[0], size_needed, NULL, NULL);
	return strTo;
}

std::wstring MRC::str2wstr(const std::string& str)
{
	if (str.empty()) return std::wstring();
	int size_needed = MultiByteToWideChar(CP_UTF8, 0, &str[0], (int)str.size(), NULL, 0);
	std::wstring wstrTo(size_needed, 0);
	MultiByteToWideChar(CP_UTF8, 0, &str[0], (int)str.size(), &wstrTo[0], size_needed);
	return wstrTo;
}








void MRC::OpenConsole()
{
	CONSOLE_SCREEN_BUFFER_INFO screenBuffer;
	AllocConsole();
	hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE);
	GetConsoleScreenBufferInfo(hStdOutput, &screenBuffer);
//	WriteInfo(TEXT("Hello"));
}
void MRC::CloseConsole()
{
	FreeConsole();
}

MRC::MRC(void) 
{
	camera = 0;
	darknoise_init_counter = 0;
	darknoise_img = NULL;
	darknoise_w = 0;
	darknoise_h = 0;
	current_img = NULL;

	std::wstringstream wss;
	OpenConsole();
	try {
		WORD error = GEVDiscovery(&dis, NULL, 200, 0);
		if (error) throw error;
	}
	catch (WORD error) {
		wss << TEXT("errno: ") << error;
		WriteInfo(wss.str());
	}
	if (dis.Count == 0) {
		MessageBox(NULL, TEXT("No CANCamGiga found."), TEXT("Error"), MB_OK);
	}
	else {
		camera = 1;
	}
}

void MRC::WriteInfo(std::wstring ws)
{
	DWORD  dwWriteByte;
	WriteConsole(hStdOutput, ws.c_str(), (DWORD)ws.size(), &dwWriteByte, NULL);
}

MRC::~MRC(void)
{
	if (darknoise_img != NULL) delete[] darknoise_img;
	if (current_img != NULL) delete[] current_img;
	if (IsStreaming()) {
		Stop();
	}
	CloseStream();
	CloseConsole();
}

std::wstring MRC::GetErrorMsg(WORD error)
{
	std::wstring ws;
	std::wstringstream wss;
	try {
		ws = status2msg.at(error);
	}
	catch (std::out_of_range) {
		wss << wformat(TEXT("UNKNOWN error:0x%04X")) % error;
		ws = wss.str();
	}
	return ws;
}
BYTE WINAPI MRC::error_callback_func(BYTE cam_nr, char* error_str)
{
	std::wstringstream wss;
	wss << error_str << std::endl;
	WriteInfo(wss.str());
	return(0);
}

BYTE WINAPI MRC::msg_callback_func(BYTE cam_nr, MESSAGECHANNEL_PARAMETER mcparam)
{
	std::wstringstream wss;
	wss << TEXT("[INFO] - Message Channel Device ") << (int)cam_nr << TEXT("->");

	switch (mcparam.EventID)
	{
	case EVENT_TRIGGER:
		wss << TEXT("trigger event");
		break;
	case EVENT_START_EXPOSUE:
		wss << TEXT("start of exposure");
		break;
	case EVENT_STOP_EXPOSUE:
		wss << TEXT("end of exposure");
		break;
	case EVENT_START_TRANSFER:
		wss << TEXT("stream channel start of transfer");
		break;
	case EVENT_STOP_TRANSFER:
		wss << TEXT("stream channel end of transfer");
		break;
	case EVENT_PRIMARY_APP_SWITCH:
		wss << TEXT("primary application switchover has been granted.");
		break;
	case EVENT_LINK_SPEED_CHANGE:
		wss << TEXT("indicates that the link speed has changed.");
		break;
	case EVENT_ACTION_LATE:
		wss << TEXT("execution of a Scheduled Action Command was late");
		break;
	default:
		if ((mcparam.EventID >= EVENT_ERROR_BEGIN) && (mcparam.EventID <= EVENT_ERROR_END)) {
			wss << TEXT("error event");
			break;
		}
		if (mcparam.EventID >= EVENT_DEVICE_SPECIFIC) {
			wss << boost::wformat(TEXT("device-specific event: 0x%04X")) % mcparam.EventID;

			break;
		}
		wss << TEXT("unknown event");
		break;
	}
	if (mcparam.DataLength) {
		std::wstringstream wss1;
		wss1 << TEXT("Event: len: ") << mcparam.DataLength << endl;
		for (int i = 0; i < mcparam.DataLength; i++) {
			wss1 << boost::wformat(TEXT("Event: data: 0x%02X")) % mcparam.Data[i] << endl;
		}
		WriteInfo(wss1.str());
	}
	WriteInfo(wss.str());
	return(0);
}

bool MRC::CloseStream(void)
{
	GEVCloseStreamChannel(camera);
	GEVClose(camera);
	IsStream = false;
	return true;
}
bool MRC::OpenStream(void)
{
	CONNECTION con;
	WORD error;
	std::wstring ws;
	std::wstringstream wss;

	con.AdapterIP = dis.param[camera- 1].AdapterIP;
	con.AdapterMask = dis.param[camera - 1].AdapterMask;
	con.IP_CANCam = dis.param[camera - 1].IP;
	con.PortCtrl = 49149;    // set 0 to port than automatic port is use
	con.PortData = 49150;    // set 0 to port than automatic port is use
	strcpy_s(con.adapter_name, sizeof(con.adapter_name), dis.param[camera - 1].adapter_name);

	try {
		error = GEVInit(camera, &con, (ERROR_CALLBACK_FUNC)error_callback_func, 0, EXCLUSIVE_ACCESS);
		if (error) {
			ws = TEXT("[ERROR] - GEVInit: ") + GetErrorMsg(error);
			throw ws;
		}
		// init xml 
		error = GEVInitXml(camera);
		if (error) {
			ws = TEXT("[ERROR] - GEVInitXml error: ") + GetErrorMsg(error);
			throw ws;
		}
		// set message channel callback function
		error = GEVSetMessageChannelCallback(camera,(MESSAGECHANNEL_CALLBACK_FUNC)msg_callback_func);
		if (error == GEV_STATUS_NOT_SUPPORTED) {
			ws=TEXT("[INFO] - Message channel not Supported\n");
			WriteInfo(ws);
		}
		// open stream channel
		error = GEVOpenStreamChannel(camera, con.AdapterIP, con.PortData, 0);
		if (error) {
			ws = TEXT("[ERROR] - GEVOpenStreamChannel error: ") + GetErrorMsg(error);
			throw ws;
		}
		// disable packet resend
		error = GEVSetPacketResend(camera, 0);
		if (error) {
			ws = TEXT("[ERROR] - GEVSetPacketResend: ") + GetErrorMsg(error);
			WriteInfo(ws);
		}
		// set stream channel packet delay to 500 (default 0)
		// dw64 = 500;   
		error = GEVSetFeatureInteger(camera, (char*)"GevSCPD", 500);
		if (error) {
			ws = TEXT("[ERROR] - GEVSetFeatureInteger(GevSCPD): ") + GetErrorMsg(error);
			WriteInfo(ws);
		}
		// find maximal packet size
		FEATURE_PARAMETER f_param;
		error = GEVGetFeatureParameter(camera, (char*)"GevSCPSPacketSize", &f_param);
		if (error) {
			ws = TEXT("[ERROR] - GevSCPSPacketSize: ") + GetErrorMsg(error);
			WriteInfo(ws);
		}
		else {
			WORD ps_max, ps_min, ps_inc, packet_size;
			// get min, max and increment of packet size
			f_param.Min == -1 ? ps_min = 0 : ps_min = (WORD)f_param.Min;
			f_param.Max == -1 ? ps_max = 20000 : ps_max = (WORD)f_param.Max;
			f_param.Inc == 1 ? ps_inc = 4 : ps_inc = (WORD)f_param.Inc;
			error = GEVTestFindMaxPacketSize(camera, &packet_size, ps_min, ps_max, ps_inc);
			if (error) {
				ws = TEXT("[ERROR] - GEVTestFindMaxPacketSize: ") + GetErrorMsg(error);
				WriteInfo(ws);
			}
			wss << TEXT("[INFO] - Find maximal packet size: ") << (int)packet_size << endl;
			WriteInfo(wss.str());
		}
	
		{

			GetMaxMinInc((char*)"Width", MAX_WIDTH, MIN_WIDTH, INC_WIDTH);
			INC_WIDTH = 4;
			GetMaxMinInc((char*)"Height", MAX_HEIGHT, MIN_HEIGHT, INC_HEIGHT);
			GetMaxMinInc((char*)"OffsetX", MAX_OFFSETX, MIN_OFFSETX, INC_OFFSETX);
			GetMaxMinInc((char*)"OffsetY", MAX_OFFSETY, MIN_OFFSETY, INC_OFFSETY); 

			if (INC_WIDTH < INC_OFFSETX) INC_WIDTH = INC_OFFSETX;
			if (INC_HEIGHT < INC_OFFSETY) INC_HEIGHT = INC_OFFSETY;
			if (MAX_OFFSETX > MAX_WIDTH - MIN_WIDTH) MAX_OFFSETX = MAX_WIDTH - MIN_WIDTH;
			if (MAX_OFFSETY > MAX_HEIGHT - MIN_HEIGHT) MAX_OFFSETY = MAX_HEIGHT - MIN_HEIGHT;

			//std::wstringstream wss;
			//wss << TEXT("Width max,min,inc: ") << (int)MAX_WIDTH << " " << (int)MIN_WIDTH << " " << (int)INC_WIDTH << endl;
			//wss << TEXT("Height max,min,inc: ") << (int)MAX_HEIGHT << " " << (int)MIN_HEIGHT << " " << (int)INC_HEIGHT << endl;
			//wss << TEXT("OffsetX max,min,inc: ") << (int)MAX_OFFSETX << " " << (int)MIN_OFFSETX << " " << (int)INC_OFFSETX << endl;
			//wss << TEXT("OffsetY max,min,inc: ") << (int)MAX_OFFSETY << " " << (int)MIN_OFFSETY << " " << (int)INC_OFFSETY << endl;
			//WriteInfo(wss.str());

		}
		

	}
	catch (wstring e) {
		WriteInfo(e);
		CloseStream();
		return false;
	}
	IsStream = true;
	return true;
}
bool MRC::IsReady()
{
	return IsStream;
}
bool MRC::IsStreaming(void)
{
	DWORD dw;
	WORD error;
	error = GEVGetFeatureCommand(camera, (char*)"AcquisitionStart", &dw);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureCommand(IsCameraAcquisition) error: ") + GetErrorMsg(error);
		throw ws;
	}
	return (bool)dw;
}

void MRC::Start(void)
{
	WORD error;
	if (IsStreaming()) return;
	error = GEVAcquisitionStart(camera, 0);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVAcquisitionStart: ") + GetErrorMsg(error);
		throw ws;
	}
}

void MRC::Stop(void)
{
	WORD error;
	if (!IsStreaming()) return;
	error = GEVAcquisitionStop(camera);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVAcquisitionStop: ") + GetErrorMsg(error);
		throw ws;
	}
}

void MRC::SetColor(void)
{
	WORD error;
	bool IsAcq;
	if (IsAcq = IsStreaming()) {
		Stop();
	}
	error = GEVSetFeatureInteger(camera, (char*)"PixelFormat", GVSP_PIX_BAYGR8);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVSetFeatureInteger: ") + GetErrorMsg(error);
		throw ws;
	}
	if (IsAcq) {
		Start();
	}
}
bool MRC::IsColor(void) 
{
	WORD error;
	INT64 value;
	error = GEVGetFeatureInteger(camera, (char*)"PixelFormat",&value);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger: ") + GetErrorMsg(error);
		throw ws;
	}
	if (value == GVSP_PIX_BAYGR8)  return true;
	return false;
}

void MRC::SetMono8(void)
{
	WORD error;
	bool IsAcq;
	if (IsAcq = IsStreaming()) {
		Stop();
	}
	error = GEVSetFeatureInteger(camera, (char*)"PixelFormat", GVSP_PIX_MONO8);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVSetFeatureInteger: ") + GetErrorMsg(error);
		throw ws;
	}
	if (IsAcq) {
		Start();
	}
}

bool MRC::SetPixelMappingMode(char* mode)
{
	WORD error;
	INT64 dw64;
	error = GEVGetFeatureInteger(camera, (char*)"PixelFormat", &dw64);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger: ") + GetErrorMsg(error);
		throw ws;
	}
	if (dw64 == GVSP_PIX_MONO12) return false;
	error = GEVSetFeatureEnumeration(camera, (char*)"PixelMappingFormat", (char*)mode, sizeof(mode));
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVSetFeatureEnumeration: ") + GetErrorMsg(error);
		throw ws;
	}
	return true;
}

bool MRC::SetPixelMappingMode(std::wstring ws)
{
	WORD error;
	INT64 dw64;
	char a_str[MAX_STRING_LENGTH];


	error = GEVGetFeatureInteger(camera, (char*)"PixelFormat", &dw64);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger: ") + GetErrorMsg(error);
		throw ws;
	}
	if (dw64 == GVSP_PIX_MONO12) return false;
	t2a((TCHAR*)ws.c_str(), a_str, MAX_STRING_LENGTH);
	error = GEVSetFeatureEnumeration(camera, (char*)"PixelMappingFormat", (char*)a_str, sizeof(a_str));
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVSetFeatureEnumeration: ") + GetErrorMsg(error);
		throw ws;
	}
	return true;
}

bool MRC::GetPixelMappingMode(std::wstring &ws)
{
	WORD error;
	INT64 dw64;
	char a_str[MAX_STRING_LENGTH];


	error = GEVGetFeatureInteger(camera, (char*)"PixelFormat", &dw64);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger: ") + GetErrorMsg(error);
		throw ws;
	}
	if (dw64 == GVSP_PIX_MONO12) return false;

	error = GEVGetFeatureEnumeration(camera, (char*)"PixelMappingFormat", (char*)a_str, sizeof(a_str));
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureEnumeration: ") + GetErrorMsg(error);
		throw ws;
	}
	ws = (boost::wformat(TEXT("%s")) % a_str).str();
	return true;
}

bool MRC::SetGlobalGain(int mode)
{
	WORD error;
	BYTE enable;
	if ((mode < 0) || (mode > 3)) return false;
	error = GEVGetFeatureEnableStatus(camera, (char*)"GlobalGain", &enable);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureEnableStatus: ") + GetErrorMsg(error);
		throw ws;
	}
	if (!enable) return false;
	error = GEVSetFeatureInteger(camera, (char*)"GlobalGain", (INT64)mode);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVSetFeatureInteger: ") + GetErrorMsg(error);
		throw ws;
	}
	darknoise_init_counter = 0;
	return true;
}

int MRC::GetGlobalGain(void)
{
	WORD error;
	INT64 dw64;

	error = GEVGetFeatureInteger(camera, (char*)"GlobalGain", &dw64);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger: ") + GetErrorMsg(error);
		throw ws;
	}
	return (int)dw64;
}

bool MRC::SetExposureTime(INT64 micro_sec)
{
	WORD error;
	BYTE enable;
	error = GEVGetFeatureEnableStatus(camera, (char*)"ExposureTime", &enable);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureEnableStatus: ") + GetErrorMsg(error);
		throw ws;
	}
	if (!enable) return false;
	error = GEVSetFeatureInteger(camera, (char*)"ExposureTime", micro_sec);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVSetFeatureInteger: ") + GetErrorMsg(error);
		throw ws;
	}
	return true;
}

INT64 MRC::GetExposureTime(void)
{
	WORD error;
	INT64 dw64;

	error = GEVGetFeatureInteger(camera, (char*)"ExposureTime", &dw64);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger: ") + GetErrorMsg(error);
		throw ws;
	}
	return dw64;
}

bool MRC::SetAutoControl(bool mode)
{
	WORD error;

	error = GEVSetFeatureBoolean(camera, (char*)"AutoControl", (DWORD)mode);
	//if (error) {
	//	wstring ws = TEXT("[ERROR] - GEVSetFeatureBoolean: ") + GetErrorMsg(error);
	//	throw ws;
	//}
	if (error) return false;
	return mode;

}

bool MRC::GetAutoControl(void)
{
	WORD error;
	DWORD mode;

	error = GEVGetFeatureBoolean(camera, (char*)"AutoControl", &mode);
	//if (error) {
	//	wstring ws = TEXT("[ERROR] - GEVSetFeatureBoolean: ") + GetErrorMsg(error);
	//	throw ws;
	//}
	if (error) return false;
	return mode;

}
void MRC::GetOffset(INT64& x, INT64& y)
{
	WORD error;
	error = GEVGetFeatureInteger(camera, (char*)"OffsetX", &x);
	if (error) {
		wstring ws  = TEXT("[ERROR] - GEVGetFeatureInteger(GetCameraOffset x) error: ") + GetErrorMsg(error);
		throw ws;
	}
	error = GEVGetFeatureInteger(camera, (char*)"OffsetY", &y);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger(GetCameraOffset y) error: ") + GetErrorMsg(error);
		throw ws;
	}
}


void MRC::GetSize(INT64& w, INT64& h)
{
	WORD error;
	error = GEVGetFeatureInteger(camera, (char*)"Width", &w);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger(Get Width) error: ") + GetErrorMsg(error);
		throw ws;
	}
	error = GEVGetFeatureInteger(camera, (char*)"Height", &h);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger(Get Height) error: ") + GetErrorMsg(error);
		throw ws;
	}
}

void MRC::GetMaxSize(INT64& w, INT64& h)
{
	WORD error;
	error = GEVGetFeatureInteger(camera, (char*)"WidthMax", &w);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger(Get WidthMax) error: ") + GetErrorMsg(error);
		throw ws;
	}
	error = GEVGetFeatureInteger(camera, (char*)"HeightMax", &h);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger(Get HeightMAx) error: ") + GetErrorMsg(error);
		throw ws;
	}
}

bool MRC::SetSize(INT64 w, INT64 h)
{
	bool IsAcq;
	WORD error;

	INT64 prev_w, prev_h, offx, offy;

	GetSize(prev_w, prev_h);
	GetOffset(offx, offy);

	if ((w + offx) > MAX_WIDTH) return false;
	if ((h + offy) > MAX_HEIGHT) return false;
	if (w < MIN_WIDTH) return false;
	if (h < MIN_HEIGHT) return false;

	if (IsAcq = IsStreaming()) {
		Stop();
	}
	if (w != prev_w) {
		error = GEVSetFeatureInteger(camera, (char*)"Width", w);
		if (error) {
			wstring ws= TEXT("[ERROR] - GEVSetFeatureInteger(SetSize W) error: ") + GetErrorMsg(error);
			throw ws;
		}
	}
	if (h != prev_h) {
		error = GEVSetFeatureInteger(camera, (char*)"Height", h);
		if (error) {
			wstring ws = TEXT("[ERROR] - GEVSetFeatureInteger(SetSize W) error: ") + GetErrorMsg(error);
			throw ws;
		}
	}
	if (IsAcq) {
		Start();
	}
	return true;
}

bool MRC::SetOffset(INT64 x, INT64 y)
{
	bool IsAcq;
	WORD error;

	INT64 prev_x, prev_y, w, h;

	GetSize(w, h);
	GetOffset(prev_x, prev_y);

	if ((w + x) > MAX_WIDTH) return false;
	if ((h + y) > MAX_HEIGHT) return false;
	if (x < MIN_OFFSETX) return false;
	if (y < MIN_OFFSETY) return false;
	if ((x % INC_OFFSETX)!=0) return false;
	if ((y % INC_OFFSETY)!=0) return false;

	if (IsAcq = IsStreaming()) {
		Stop();
	}
	if (x != prev_x) {
		error = GEVSetFeatureInteger(camera, (char*)"OffsetX", x);
		if (error) {
			wstring ws = TEXT("[ERROR] - GEVSetFeatureInteger(SetOffset x) error: ") + GetErrorMsg(error);
			throw ws;
		}
	}
	if (y != prev_y) {
		error = GEVSetFeatureInteger(camera, (char*)"OffsetY", y);
		if (error) {
			wstring ws = TEXT("[ERROR] - GEVSetFeatureInteger(SetOffset y) error: ") + GetErrorMsg(error);
			throw ws;
		}
	}
	if (IsAcq) {
		Start();
	}
	return true;
}

bool MRC::SetGainG(int green)
{
	WORD error = GEVSetFeatureInteger(camera, (char*)"GainGreen", (INT64)green);
	//if (error) {
	//	wstring ws = TEXT("[ERROR] - GEVSetFeatureInteger(SetGainG) error: ") + GetErrorMsg(error);
	//	throw ws;
	//}
	if (error) return false;
	return true;
}
int MRC::GetGainG(void)
{
	INT64 dw64;
	WORD error = GEVGetFeatureInteger(camera, (char*)"GainGreen", &dw64);
	//if (error) {
	//	wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger(GetGainG) error: ") + GetErrorMsg(error);
	//	throw ws;
	//}
	if (error) return 0;
	return (int)dw64;
}

bool MRC::SetGainR(int red)
{
	WORD error = GEVSetFeatureInteger(camera, (char*)"GainRed", (INT64)red);
	//if (error) {
	//	wstring ws = TEXT("[ERROR] - GEVSetFeatureInteger(SetGainR) error: ") + GetErrorMsg(error);
	//	throw ws;
	//}
	if (error) return false;
	return true;
}
int MRC::GetGainR(void)
{
	INT64 dw64;
	WORD error = GEVGetFeatureInteger(camera, (char*)"GainRed", &dw64);
	//if (error) {
	//	wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger(GetGainR) error: ") + GetErrorMsg(error);
	//	throw ws;
	//}
	if (error) return 0;
	return (int)dw64;
}

bool MRC::SetGainB(int blue)
{
	WORD error = GEVSetFeatureInteger(camera, (char*)"GainBlue", (INT64)blue);
	//if (error) {
	//	wstring ws = TEXT("[ERROR] - GEVSetFeatureInteger(SetGainB) error: ") + GetErrorMsg(error);
	//	throw ws;
	//}
	if (error) return false;
	return true;
}
int MRC::GetGainB(void)
{
	INT64 dw64;
	WORD error = GEVGetFeatureInteger(camera, (char*)"GainBlue", &dw64);
	//if (error) {
	//	wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger(GetGainB) error: ") + GetErrorMsg(error);
	//	throw ws;
	//}
	if (error) return 0;
	return (int)dw64;
}

bool MRC::SetFPS(int64 fps)
{
	WORD error = GEVSetFeatureInteger(camera, (char*)"FPS", (INT64)fps);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger(FPS) error: ") + GetErrorMsg(error);
		WriteInfo(ws);
		return false;
	}
	return true;
}

int MRC::GetFPS(void)
{
	INT64 dw64;
	WORD error = GEVGetFeatureInteger(camera, (char*)"FPS", &dw64);
	if (error) return 0;
	return (int)dw64;
}

bool MRC::SetRowFPN(int fpn)
{
	WORD error = GEVSetFeatureBoolean(camera, (char*)"RowFPN", (DWORD)fpn);
	if (error) return false;
	return true;
}

bool MRC::GetRowFPN(void)
{
	DWORD dw;
	WORD error = GEVGetFeatureBoolean(camera, (char*)"RowFPN", &dw);
	if (error) return 0;
	return (int)dw;
}

bool MRC::SetFT_TIMER(INT64 ftt)
{
	WORD error = GEVSetFeatureInteger(camera, (char*)"FT_TIMER", (INT64)ftt);
	if (error) return false;
	return true;
}

INT64 MRC::GetFT_TIMER(void)
{
	INT64 dw64;
	WORD error = GEVGetFeatureInteger(camera, (char*)"FT_TIMER", &dw64);
	if (error) return 0;
	return dw64;
}



INT64 MRC::GetImageSize(void)
{
	INT64 dw64;
	WORD error = GEVGetFeatureInteger(camera, (char*)"PayloadSize", &dw64);
	if (error) {
		wstring ws = TEXT("[ERROR] - GEVGetFeatureInteger(PayloadSize) error: ") + GetErrorMsg(error);
		throw ws;
	}
	return dw64;
}

bool MRC::SetExposureTimeRaw(INT64 etr)
{
	WORD error = GEVSetFeatureInteger(camera, (char*)"ExposureTimeRaw", (INT64)etr);
	if (error) return false;
	return true;
}

INT64 MRC::GetExposureTimeRaw(void)
{
	INT64 dw64;
	WORD error = GEVGetFeatureInteger(camera, (char*)"ExposureTimeRaw", &dw64);
	if (error) return 0;
	return dw64;
}

 void MRC::GetHistogram(CameraImage& ci, MRC_COLOR_HISTOGRAM& hist)
 {

	 int w, h, i, j, offset_j, r_max, g_max, b_max, pos;
	 for (i = 0; i < 256; i++) {
		 hist.R[i] = hist.G[i] = hist.B[i] = 0;
	 }
	 w = (int)ci.w;
	 h = (int)ci.h;
	 r_max = g_max = b_max = 0;

	 for (j = 0; j < h; j += 2) {
		 offset_j = j * w;
		 for (i = 0; i < w; i += 2) {
			 pos = ci.img[i + offset_j];
			 if (++hist.G[pos] > g_max) g_max = hist.G[pos];
		 }
	 }
	 for (j = 1; j < h; j += 2) {
		 offset_j = j * w;
		 for (i = 1; i < w; i += 2) {
			 pos = ci.img[i + offset_j];
			 if (++hist.G[pos] > g_max) g_max = hist.G[pos];
		 }
	 }
	 for (j = 0; j < h; j += 2) {
		 offset_j = j * w;
		 for (i = 1; i < w; i += 2) {
			 pos = ci.img[i + offset_j];
			 if (++hist.R[pos] > r_max) r_max = hist.R[pos];
		 }
	 }
	 for (j = 1; j < h; j += 2) {
		 offset_j = j * w;
		 for (i = 0; i < w; i += 2) {
			 pos = ci.img[i + offset_j];
			 if (++hist.B[pos] > b_max) b_max = hist.B[pos];
		 }
	 }
	 for (i = 0; i < 256; i++) {
		 if (r_max != 0) {
			 hist.R[i] = hist.R[i] * 100 / r_max;
		 }
		 if (g_max != 0) {
			 hist.G[i] = hist.G[i] * 100 / g_max;
		 }
		 if (b_max != 0) {
			 hist.B[i] = hist.B[i] * 100 / b_max;
		 }
	 }
 }

 bool MRC::GetMaxMinInc(char* feature_name, INT64& max, INT64& min, INT64& inc)
 {
	 FEATURE_PARAMETER f_param;
	 WORD error;
	 std::wstring ws;
	 error = GEVGetFeatureParameter(camera, feature_name, &f_param);
	 if (error) {
		 ws = TEXT("[ERROR] - GetMaxMinInc: ") + GetErrorMsg(error);
		 WriteInfo(ws);
		 return false;
	 }
	 max = f_param.Max;
	 min = f_param.Min;
	 inc = f_param.Inc;
	 return true;
 }
bool SaveImg(IMAGE_HEADER& imh, BYTE* ppixel)
{
	int h, w;
	w = imh.SizeX;
	h = imh.SizeY;
	cv::Mat img = cv::Mat(h, w, CV_8UC1, ppixel);
	cv::Mat color;
	cv::Mat gray;
	cv::cvtColor(img, color, cv::COLOR_BayerGR2RGB);
	cv::cvtColor(color, gray, cv::COLOR_RGB2GRAY);
	cv::imwrite("color.bmp", color);
	cv::imwrite("gray.bmp", gray);
	cv::imwrite("orig_gray.bmp", img);

	return true;
}

bool MRC::TempProc(void)
{
	IMAGE_HEADER img_header;
	DWORD img_size;
	BYTE* img=NULL;
	wstringstream wss;
	WORD error;
	try {
		SetOffset(0, 0);
		SetSize(1280, 960);
		SetMono8();
		SetPixelMappingMode((char*)"HighBits");
		SetAutoControl(false);
		SetGlobalGain(1);
		SetExposureTime(4000);
		SetGainR(48);
		SetGainG(32);
		SetGainB(48);

		img_size = (DWORD)GetImageSize();
		img = new BYTE[img_size];
		Start();
		error = GEVGetImageBuffer(camera, &img_header, img);
		if ((error) && (error != GEV_STATUS_GRAB_ERROR)) {
			wstring ws = TEXT("[ERROR] - GEVGetImage: ") + GetErrorMsg(error);
			throw ws;
		}
		else {
			wss << TEXT("[INFO] - Image: ") << img_header.FrameCounter << TEXT(", Missing Packets: ") << img_header.MissingPacket << endl;
			WriteInfo(wss.str());
		}
		Stop();

		Start();
		error = GEVGetImageBuffer(camera, &img_header, img);
		if ((error) && (error != GEV_STATUS_GRAB_ERROR)) {
			wstring ws = TEXT("[ERROR] - GEVGetImage: ") + GetErrorMsg(error);
			throw ws;
		}
		else {
			wss << TEXT("[INFO] - Image: ") << img_header.FrameCounter << TEXT(", Missing Packets: ") << img_header.MissingPacket << endl;
			WriteInfo(wss.str());
		}
		Stop();

		SaveImg(img_header, img);
		delete [] img;
	}
	catch (wstring ws) {
		if (img != NULL) delete [] img;
		WriteInfo(ws);
		CloseStream();
		return false;
	}

	return true;
}

MRC::DlgSelect::DlgSelect()
{
	CamPtr = NULL;
	hWndParent = NULL;
	lb.reset();
}

MRC::DlgSelect::~DlgSelect()
{

}

INT_PTR MRC::DlgSelect::Open(HWND hWndParent, MRC& mrc)
{
	INT_PTR iptr;
	hWndParent = hWndParent;
	CamPtr = &mrc;
	if (CamPtr->dis.Count == 0) return false;
	iptr = DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_MRC_CAM_LIST), hWndParent, CallbackProc, (LPARAM)this);
	return iptr;
}

bool MRC::DlgSelect::InitCtrl(HWND hDlg)
{
	std::wstringstream wss, wss1;
	lb.reset(new DlgListBox(hDlg, IDC_CAMERA_LIST));

	for (int i = 0; i < CamPtr->dis.Count; i++) {
		wss.str(TEXT(""));
		wss.clear(std::stringstream::goodbit);

		wss << boost::wformat(TEXT("%-20s")) % (char*)CamPtr->dis.param[i].manuf;
		wss << boost::wformat(TEXT("%-20s")) % (char*)CamPtr->dis.param[i].model;
		wss << boost::wformat(TEXT("%-20s")) % (char*)CamPtr->dis.param[i].version;

		wss1.str(TEXT(""));
		wss1.clear(std::stringstream::goodbit);
		wss1 << boost::wformat(TEXT("%d.%d.%d.%d"))
			% (unsigned char)((CamPtr->dis.param[i].IP) & 0xff)
			% (unsigned char)((CamPtr->dis.param[i].IP >> 8) & 0xff)
			% (unsigned char)((CamPtr->dis.param[i].IP >> 16) & 0xff)
			% (unsigned char)((CamPtr->dis.param[i].IP >> 24) & 0xff);
		wss << boost::wformat(TEXT("%-20s")) % wss1.str();

		wss1.str(TEXT(""));
		wss1.clear(std::stringstream::goodbit);
		wss1 << boost::wformat(TEXT("%x:%x:%x:%x:%x:%x"))
			% (unsigned char)(CamPtr->dis.param[i].Mac[0])
			% (unsigned char)(CamPtr->dis.param[i].Mac[1])
			% (unsigned char)(CamPtr->dis.param[i].Mac[2])
			% (unsigned char)(CamPtr->dis.param[i].Mac[3])
			% (unsigned char)(CamPtr->dis.param[i].Mac[4])
			% (unsigned char)(CamPtr->dis.param[i].Mac[5]);
		wss << boost::wformat(TEXT("%-20s")) % wss1.str();

		lb->add(wss.str());
	}
	lb->Set(0);
	CamPtr->camera = 1;

	return true;
}

void MRC::DlgSelect::EndCtrl(HWND hDlg)
{

}

bool MRC::DlgSelect::DlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM)
{
	int wmId, wmEvent;

	switch (message)
	{
	case WM_INITDIALOG:
		if (InitCtrl(hDlg)) return (INT_PTR)true;
		EndDialog(hDlg, false);
		return (INT_PTR)true;
	case WM_COMMAND:
		wmId = LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		switch (wmId) {
		case IDC_CAMERA_LIST:
			CamPtr->camera = lb->Get() + 1;
			return (INT_PTR)true;
		case IDOK:
			CamPtr->OpenStream();
			CamPtr->GetCameraParameter(CamPtr->CameraParam);
			//CamPtr->Start();
			EndCtrl(hDlg);
			EndDialog(hDlg, true);
			return (INT_PTR)true;
		case IDCANCEL:
			EndCtrl(hDlg);
			EndDialog(hDlg, false);
			return (INT_PTR)true;
		}
		return (INT_PTR)true;
	}
	return false;
}

INT_PTR CALLBACK MRC::DlgSelect::CallbackProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	DlgSelect* thisPtr = NULL;

	switch (message) {
	case WM_INITDIALOG:
		thisPtr = (DlgSelect*)lParam;
		SetWindowLongPtr(hDlg, DWLP_USER, (LONG_PTR)lParam);
		break;
	case WM_DESTROY:
		thisPtr = (DlgSelect*)GetWindowLongPtr(hDlg, DWLP_USER);
		SetWindowLongPtr(hDlg, DWLP_USER, (LONG_PTR)0);
		break;
	default:
		thisPtr = (DlgSelect*)GetWindowLongPtr(hDlg, DWLP_USER);
		break;
	}
	if (thisPtr) {
		return thisPtr->DlgProc(hDlg, message, wParam, lParam);
	}
	return false;
}

bool MRC::DialogSelect(HWND hWnd)
{
	DlgSelect ds;
	return ds.Open(hWnd, *this);
}
bool MRC::DialogRegion(HWND hWnd)
{
	InputRegion sc;
	return sc.Open(hWnd, *this);
}
bool MRC::DialogParameters(HWND hWnd)
{
	Parameters pm;
	return pm.Open(hWnd, *this);
}


#define ID_SETTINGCAMERA_TIMER 10001
#define SETTINGCAMERA_INTERVAL 10
#define BMP_OFFSETX 0
#define BMP_OFFSETY 200

MRC::InputRegion::InputRegion()
{
	//img = NULL;
	//img_size = 0;	
	cam = NULL;
	hWndParent = NULL;

	start = end = { 0,0 };
	hCursorSIZENESW = LoadCursor(NULL, IDC_SIZENESW);
	hCursorSIZENWSE = LoadCursor(NULL, IDC_SIZENWSE);
	hCursorArrow = LoadCursor(NULL, IDC_ARROW);
}

MRC::InputRegion::~InputRegion()
{

}


void MRC::InputRegion::SetInputOffset(void)
{
	int lx, ly, rx, ry;

	lx = start.x - BMP_OFFSETX;
	ly = start.y - BMP_OFFSETY;

	lx = lx - lx % (int)cam->INC_OFFSETX;
	if (lx < (int)cam->MIN_OFFSETX) lx = (int)cam->MIN_OFFSETX;
	if (lx > (int)cam->MAX_OFFSETX) lx = (int)cam->MAX_OFFSETX;

	ly = ly - ly % (int)cam->INC_OFFSETY;
	if (ly < (int)cam->MIN_OFFSETY) ly = (int)cam->MIN_OFFSETY;
	if (ly > (int)cam->MAX_OFFSETY) ly = (int)cam->MAX_OFFSETY;

	rx = lx + (int)cam->MIN_WIDTH;
	if (rx >= (int)cam->MAX_WIDTH) rx = (int)cam->MAX_WIDTH;
	ry = ly + (int)cam->MIN_HEIGHT;
	if (ry >= (int)cam->MAX_HEIGHT) ry = (int)cam->MAX_HEIGHT;



	sp_offsetx->Set(lx);
	sp_offsety->Set(ly);
	sp_width->Set(rx - lx);
	sp_height->Set(ry - ly);

}
void MRC::InputRegion::SetInputSize(void)
{
	int lx, ly, rx, ry, w, h;
	int flag = 1;
	if (start.x < end.x) {
		lx = start.x - BMP_OFFSETX;
		rx = end.x - BMP_OFFSETX;
	}
	else {
		flag = flag * -1;
		rx = start.x - BMP_OFFSETX;
		lx = end.x - BMP_OFFSETX;
	}
	if (start.y < end.y) {
		ly = start.y - BMP_OFFSETY;
		ry = end.y - BMP_OFFSETY;
	}
	else {
		ry = start.y - BMP_OFFSETY;
		ly = end.y - BMP_OFFSETY;
		flag = flag * -1;
	}
	if (flag == 1) {
		SetCursor(hCursorSIZENWSE);
	}
	else {
		SetCursor(hCursorSIZENESW);
	}

	lx = lx - lx % (int)cam->INC_OFFSETX;
	if (lx < (int)cam->MIN_OFFSETX) lx = (int)cam->MIN_OFFSETX;
	if (lx > (int)cam->MAX_OFFSETX) lx = (int)cam->MAX_OFFSETX;

	ly = ly - ly % (int)cam->INC_OFFSETY;
	if (ly < (int)cam->MIN_OFFSETY) ly = (int)cam->MIN_OFFSETY;
	if (ly > (int)cam->MAX_OFFSETY) ly = (int)cam->MAX_OFFSETY;

	w = (rx - lx);
	w = w - w % (int)cam->INC_WIDTH;
	h = (ry - ly);
	h = h - h % (int)cam->INC_HEIGHT;

	if ((lx + w) > (int)cam->MAX_WIDTH) {
		w = (int)cam->MAX_WIDTH - lx;
	}
	if ((ly + h) > (int)cam->MAX_HEIGHT) {
		h = (int)cam->MAX_HEIGHT - ly;
	}

	sp_offsetx->Set(lx);
	sp_offsety->Set(ly);
	sp_width->Set(w);
	sp_height->Set(h);
}
void MRC::InputRegion::SP_SetOffsetX(void)
{
	int x, w;
	x = sp_offsetx->Get();
	w = sp_width->Get();
	if ((x + w) > (int)cam->MAX_WIDTH) {
		sp_width->Set((int)cam->MAX_WIDTH - x);
	}
}

void MRC::InputRegion::SP_SetOffsetY(void)
{
	int y, h;
	y = sp_offsety->Get();
	h = sp_height->Get();
	if ((y + h) > (int)cam->MAX_HEIGHT) {
		sp_height->Set((int)cam->MAX_HEIGHT - y);
	}
}

void MRC::InputRegion::SP_SetWidth(void)
{
	int x, w;
	x = sp_offsetx->Get();
	w = sp_width->Get();
	if ((x + w) > (int)cam->MAX_WIDTH) {
		x = (int)cam->MAX_WIDTH - w;
		if (x > (int)cam->MIN_OFFSETX) {
			sp_offsetx->Set(x);
		}
		else {
			sp_offsetx->Set((int)cam->MIN_OFFSETX);
			sp_width->Set((int)cam->MAX_WIDTH - (int)cam->MIN_OFFSETX);
		}
	}
}

void MRC::InputRegion::SP_SetHeight(void)
{
	int y, h;
	y = sp_offsety->Get();
	h = sp_height->Get();
	if ((y + h) > (int)cam->MAX_HEIGHT) {
		y = (int)cam->MAX_HEIGHT - h;
		if (y > (int)cam->MIN_OFFSETY) {
			sp_offsety->Set(y);
		}
		else {
			sp_offsety->Set((int)cam->MIN_OFFSETY);
			sp_height->Set((int)cam->MAX_HEIGHT - (int)cam->MIN_OFFSETY);
		}
	}
}

INT_PTR MRC::InputRegion::Open(HWND hWnd, MRC& camera)
{
	INT_PTR iptr;
	hWndParent = hWnd;
	cam = &camera;
	iptr = DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_MRC_REGION), hWndParent, CallbackProc, (LPARAM)this);
	return iptr;
}



void MRC::InputRegion::EndCtrl(HWND hDlg)
{
	SetCursor(hCursorArrow);
	KillTimer(hDlg, ID_SETTINGCAMERA_TIMER);
	//cam->Stop();

}

BOOL MRC::InputRegion::DlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	static bool LButtonDown = false;

	std::wstringstream wss;

	cv::Mat org;
	cv::Mat color;
	MRC_COLOR_HISTOGRAM hist;
	static bool busy = false;
	bool mode;

	switch (message)
	{
	case WM_INITDIALOG:
		InitCtrl(hDlg);
		return (INT_PTR)TRUE;
	case WM_LBUTTONDOWN:
		start = end = MAKEPOINTS(lParam);
		if ((start.x > BMP_OFFSETX) && (start.y > BMP_OFFSETY)) {
			LButtonDown = true;
			SetCursor(hCursorSIZENWSE);
			SetInputOffset();

			SetCapture(hDlg);
		}
		return (INT_PTR)TRUE;
	case WM_MOUSEMOVE:
		if (LButtonDown) {
			end = MAKEPOINTS(lParam);
			SetInputSize();
		}
		return (INT_PTR)TRUE;
	case WM_LBUTTONUP:
		if (LButtonDown) {
			LButtonDown = false;
			SetCursor(hCursorArrow);
			ReleaseCapture();
		}
		return (INT_PTR)TRUE;
	case WM_HSCROLL:
		if ((HWND)lParam == sl_r->hWnd) {
			cam->SetGainR(sl_r->Get());
		}
		if ((HWND)lParam == sl_g->hWnd) {
			cam->SetGainG(sl_g->Get());
		}
		if ((HWND)lParam == sl_b->hWnd) {
			cam->SetGainB(sl_b->Get());
		}
		if ((HWND)lParam == sl_globalgain->hWnd) {
			cam->SetGlobalGain(sl_globalgain->Get());
		}
		if ((HWND)lParam == sl_exposure->hWnd) {
			cam->SetExposureTime(sl_exposure->Get());
		}
		return (INT_PTR)TRUE;
	case WM_VSCROLL:
		if ((HWND)lParam == sp_offsetx->hWnd && LOWORD(wParam) == SB_THUMBPOSITION) {
			SP_SetOffsetX();
		}

		if ((HWND)lParam == sp_offsety->hWnd && LOWORD(wParam) == SB_THUMBPOSITION) {
			SP_SetOffsetY();
		}

		if ((HWND)lParam == sp_width->hWnd && LOWORD(wParam) == SB_THUMBPOSITION) {
			SP_SetWidth();
		}

		if ((HWND)lParam == sp_height->hWnd && LOWORD(wParam) == SB_THUMBPOSITION) {
			SP_SetHeight();
		}

		return (INT_PTR)TRUE;
	case WM_TIMER:
		if (busy) return (INT_PTR)TRUE;
		busy = true;

		cam->CaptureImage(ci, L"CaptureImage");

		if (rb_color->Get() == true) {
			org = cv::Mat((int)ci.h, (int)ci.w, CV_8UC1, ci.img);
			cv::cvtColor(org, color, cv::COLOR_BayerGR2RGB);
			bmp->copy(color.data, (int)ci.w, (int)ci.h, 24);
		}
		else {
			bmp->copy(ci.img, (int)ci.w, (int)ci.h, 8);
		}
		if (ckb_histogram->Get() == true) {
			cam->GetHistogram(ci, hist);
			bmp->histogram(0, 100, hist.R, RGB(255, 0, 0), SRCCOPY);
			bmp->histogram(0, 100, hist.G, RGB(0, 255, 0), SRCPAINT);
			bmp->histogram(0, 100, hist.B, RGB(0, 0, 255), SRCPAINT);
		}

		bmp->rect(sp_offsetx->Get(), sp_offsety->Get(), sp_width->Get(), sp_height->Get(), RGB(255, 0, 0));
		bmp->refresh();
		busy = false;
		return (INT_PTR)TRUE;
	case WM_COMMAND:
		wmId = LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		switch (wmId) {
		case IDC_CHECK_HISTOGRAM:

			return (INT_PTR)true;
		case IDC_CHECK_AUTO:
			mode = ckb_auto->Get();
			cam->SetAutoControl(mode);
			cam->CameraParam.AutoControl = mode;
			return (INT_PTR)true;
		case IDC_COMBO_BITS:
			if (wmEvent == CBN_SELCHANGE) {
				cam->SetPixelMappingMode(cl_bits->GetString());
			}
			return (INT_PTR)true;
		case IDC_RADIO_COLOR:
			if (wmEvent == BN_CLICKED) {
				cam->SetColor();
				cam->CameraParam.Color = rb_color->Get();
			}
			return (INT_PTR)true;
		case IDC_RADIO_MONO:
			if (wmEvent == BN_CLICKED) {
				cam->SetMono8();
				cam->CameraParam.Color = rb_color->Get();
			}
			return (INT_PTR)true;
		case IDOK:
			cam->GetCameraParameter(cam->CameraParam);
			cam->CameraParam.OffsetX = sp_offsetx->Get();
			cam->CameraParam.OffsetY = sp_offsety->Get();
			cam->CameraParam.w = sp_width->Get();
			cam->CameraParam.h = sp_height->Get();
			//cam->CamParam.Color = rb_color->Get();
			cam->CameraParam.FPS = 500;
			cam->SetCameraParameter(cam->CameraParam);
			//cam->GetCameraParameter(cam->CameraParam);
			EndCtrl(hDlg);
			EndDialog(hDlg, wmId);
			return (INT_PTR)true;
		case IDCANCEL:
			cam->SetCameraParameter(BackUpCamParam);
			cam->GetCameraParameter(cam->CameraParam);
			EndCtrl(hDlg);
			EndDialog(hDlg, wmId);
			return (INT_PTR)true;
		}
		return (INT_PTR)false;
	}
	return false;
}
void MRC::InputRegion::InitCtrl(HWND hDlg)
{
	RECT rc;
	std::wstring ws;

	cam->Stop();
	cam->GetCameraParameter(cam->CameraParam);
	cam->GetCameraParameter(BackUpCamParam);
	int offx, offy, width, height;
	offx = (int)cam->CameraParam.OffsetX;
	offy = (int)cam->CameraParam.OffsetY;
	width = (int)cam->CameraParam.w;
	height = (int)cam->CameraParam.h;

	cam->SetOffset(0, 0);
	cam->SetSize(cam->MAX_WIDTH, cam->MAX_HEIGHT);
	cam->SetFPS(250);
	cam->GetCameraParameter(cam->CameraParam);

	cam->Start();


	GetWindowRect(hDlg, &rc);
	rc.right = rc.left + (LONG)cam->MAX_WIDTH;
	rc.bottom = rc.top + (LONG)cam->MAX_HEIGHT + BMP_OFFSETY;
	AdjustWindowRect(&rc, WS_OVERLAPPEDWINDOW, false);
	if (rc.top < 0) {
		rc.bottom = rc.bottom - rc.top;
		rc.top = 0;
	}
	MoveWindow(hDlg, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, true);
	GetClientRect(hDlg, &rc);
	btn_OK.reset(new DlgButton(hDlg, IDOK));

	bmp.reset(new DlgBitmap(hDlg, IDC_CAM_BITMAP, (int)cam->MAX_WIDTH, (int)cam->MAX_HEIGHT));
	bmp->fill(WHITENESS);
	bmp->move(BMP_OFFSETX, BMP_OFFSETY);
	//	bmp->refresh();

	sl_r.reset(new DlgSlider(hDlg, IDC_SLIDER_R, 0, 255, cam->GetGainR(), 10, 1));
	sl_g.reset(new DlgSlider(hDlg, IDC_SLIDER_G, 0, 255, cam->GetGainG(), 10, 1));
	sl_b.reset(new DlgSlider(hDlg, IDC_SLIDER_B, 0, 255, cam->GetGainB(), 10, 1));
	sl_globalgain.reset(new DlgSlider(hDlg, IDC_SLIDER_GLOBALGAIN, 0, 3, cam->GetGlobalGain(), 1, 1));
	sl_exposure.reset(new DlgSlider(hDlg, IDC_SLIDER_EXPOSURE, 0, 20000, (int)cam->GetExposureTime(), 1000, 100));
	ckb_histogram.reset(new DlgCheckbox(hDlg, IDC_CHECK_HISTOGRAM));
	ckb_histogram->Set(true);
	ckb_auto.reset(new DlgCheckbox(hDlg, IDC_CHECK_AUTO));

	ckb_auto->Set(cam->GetAutoControl());

	rb_color.reset(new DlgRadioButton(hDlg, IDC_RADIO_COLOR));
	rb_mono.reset(new DlgRadioButton(hDlg, IDC_RADIO_MONO));

	if (cam->IsColor()) {
		rb_color->Set(true);
	}
	else {
		rb_mono->Set(true);
	}

	cl_bits.reset(new DlgComboList(hDlg, IDC_COMBO_BITS));

	BOOST_FOREACH(auto modeString, cam->PixelMappingModeString) {
		cl_bits->add(modeString);
	}
	cam->GetPixelMappingMode(ws);
	cl_bits->Set(ws);

	eb_offsetx.reset(new DlgEditBox(hDlg, IDC_EDIT_OFFSETX));
	eb_offsety.reset(new DlgEditBox(hDlg, IDC_EDIT_OFFSETY));
	eb_width.reset(new DlgEditBox(hDlg, IDC_EDIT_WIDTH));
	eb_height.reset(new DlgEditBox(hDlg, IDC_EDIT_HEIGHT));

	sp_offsetx.reset(new DlgSpin(hDlg, IDC_SPIN_OFFSETX));
	sp_offsety.reset(new DlgSpin(hDlg, IDC_SPIN_OFFSETY));
	sp_width.reset(new DlgSpin(hDlg, IDC_SPIN_WIDTH));
	sp_height.reset(new DlgSpin(hDlg, IDC_SPIN_HEIGHT));


	sp_offsetx->SetBuddy(eb_offsetx->hWnd);
	sp_offsety->SetBuddy(eb_offsety->hWnd);
	sp_width->SetBuddy(eb_width->hWnd);
	sp_height->SetBuddy(eb_height->hWnd);


	sp_width->SetMinMax((int)cam->MIN_WIDTH, (int)cam->MAX_WIDTH);
	sp_height->SetMinMax((int)cam->MIN_HEIGHT, (int)cam->MAX_HEIGHT);
	sp_offsetx->SetMinMax((int)cam->MIN_OFFSETX, (int)cam->MAX_OFFSETX);
	sp_offsety->SetMinMax((int)cam->MIN_OFFSETY, (int)cam->MAX_OFFSETY);

	sp_width->Set(width);
	sp_height->Set(height);
	sp_offsetx->Set(offx);
	sp_offsety->Set(offy);

	sp_width->SetAccel((int)cam->INC_WIDTH);
	sp_height->SetAccel((int)cam->INC_HEIGHT);
	sp_offsetx->SetAccel((int)cam->INC_OFFSETX);
	sp_offsety->SetAccel((int)cam->INC_OFFSETY);

	SetTimer(hDlg, ID_SETTINGCAMERA_TIMER, SETTINGCAMERA_INTERVAL, NULL);
}
INT_PTR CALLBACK MRC::InputRegion::CallbackProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	InputRegion* thisPtr = NULL;

	switch (message) {
	case WM_INITDIALOG:
		thisPtr = (InputRegion*)lParam;
		SetWindowLongPtr(hDlg, DWLP_USER, (LONG_PTR)lParam);
		break;
	case WM_DESTROY:
		thisPtr = (InputRegion*)GetWindowLongPtr(hDlg, DWLP_USER);
		SetWindowLongPtr(hDlg, DWLP_USER, (LONG_PTR)0);
		break;
	default:
		thisPtr = (InputRegion*)GetWindowLongPtr(hDlg, DWLP_USER);
		break;
	}
	if (thisPtr) {
		return thisPtr->DlgProc(hDlg, message, wParam, lParam);
	}
	return false;
}

MRC::Parameters::Parameters()
{
	cam = NULL;
	hWndParent = NULL;
}

MRC::Parameters::~Parameters()
{

}


INT_PTR MRC::Parameters::Open(HWND hWnd, MRC& camera)
{
	INT_PTR iptr;
	hWndParent = hWnd;
	cam = &camera;
	iptr = DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_MRC_PARAMETER), hWndParent, CallbackProc, (LPARAM)this);
	return iptr;
}

void MRC::Parameters::GetCamParam(MRC_CAMERA_PARAM& p)
{
	p.R = cam->GetGainR();
	p.G = cam->GetGainG();
	p.B = cam->GetGainB();

	p.AutoControl = cam->GetAutoControl();
	p.GlobalGain = cam->GetGlobalGain();
	p.ExposureTime = cam->GetExposureTime();

	p.Color = cam->IsColor();
	if (!cam->GetPixelMappingMode(p.mode)) {
		cam->SetMono8();
		cam->GetPixelMappingMode(p.mode);
	}
	cam->GetSize(p.w, p.h);
	cam->GetOffset(p.OffsetX, p.OffsetY);
}
void MRC::Parameters::SetCamParam(MRC_CAMERA_PARAM& p)
{
	cam->SetGainR(p.R);
	cam->SetGainG(p.G);
	cam->SetGainB(p.B);
	cam->SetAutoControl(p.AutoControl);
	cam->SetGlobalGain(p.GlobalGain);
	cam->SetExposureTime(p.ExposureTime);

	if (p.Color) {
		cam->SetColor();
	}
	else {
		cam->SetMono8();
	}
	cam->SetPixelMappingMode(p.mode);

	cam->SetSize(p.w, p.h);
	cam->SetOffset(p.OffsetX, p.OffsetY);
}

void MRC::Parameters::EndCtrl(HWND hDlg)
{

	KillTimer(hDlg, ID_SETTINGCAMERA_TIMER);
	//cam->Save();
	
	//cam->Stop();
}

BOOL MRC::Parameters::DlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	static bool LButtonDown = false;

	std::wstringstream wss;
	std::wstring ws;

	cv::Mat org;
	cv::Mat color;
	MRC_COLOR_HISTOGRAM hist;
	static bool busy = false;
	bool mode;

	switch (message)
	{
	case WM_INITDIALOG:
		InitCtrl(hDlg);
		return (INT_PTR)TRUE;

	case WM_HSCROLL:
		if ((HWND)lParam == sl_r->hWnd) {
			cam->SetGainR(sl_r->Get());
		}
		if ((HWND)lParam == sl_g->hWnd) {
			cam->SetGainG(sl_g->Get());
		}
		if ((HWND)lParam == sl_b->hWnd) {
			cam->SetGainB(sl_b->Get());
		}
		if ((HWND)lParam == sl_globalgain->hWnd) {
			cam->SetGlobalGain(sl_globalgain->Get());
		}
		if ((HWND)lParam == sl_exposure->hWnd) {
			cam->SetExposureTime(sl_exposure->Get());
		}
		if ((HWND)lParam == sl_fps->hWnd) {
			cam->SetFPS(sl_fps->Get());
		}
		return (INT_PTR)TRUE;
	//case WM_VSCROLL:


	//	return (INT_PTR)TRUE;
	case WM_TIMER:

		if (busy) return (INT_PTR)TRUE;
		busy = true;
		GEVGetImageFPS(cam->camera, &cam->fps);
		ws = (boost::wformat(L"fps: %5.2f") % cam->fps).str();
		lb_fps->SetText(ws);

		cam->CaptureImage(ci, L"CaptureImage");

		if (rb_color->Get() == true) {
			org = cv::Mat((int)ci.h, (int)ci.w, CV_8UC1, ci.img);
			cv::cvtColor(org, color, cv::COLOR_BayerGR2RGB);
			bmp->copy(color.data, (int)ci.w, (int)ci.h, 24);
		}
		else {
			bmp->copy(ci.img, (int)ci.w, (int)ci.h, 8);
		}
		if (ckb_histogram->Get() == true) {
			cam->GetHistogram(ci, hist);
			bmp->histogram(0, 100, hist.R, RGB(255, 0, 0), SRCCOPY);
			bmp->histogram(0, 100, hist.G, RGB(0, 255, 0), SRCPAINT);
			bmp->histogram(0, 100, hist.B, RGB(0, 0, 255), SRCPAINT);
		}

		bmp->refresh();
		busy = false;
		return (INT_PTR)TRUE;
	case WM_COMMAND:
		wmId = LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		switch (wmId) {
		case IDC_CHECK_HISTOGRAM:

			return (INT_PTR)true;
		case IDC_CHECK_AUTO:
			mode = ckb_auto->Get();
			cam->SetAutoControl(mode);
			cam->CameraParam.AutoControl = mode;
			return (INT_PTR)true;
		case IDC_COMBO_BITS:
			if (wmEvent == CBN_SELCHANGE) {
				cam->SetPixelMappingMode(cl_bits->GetString());
			}
			return (INT_PTR)true;
		case IDC_RADIO_COLOR:
			if (wmEvent == BN_CLICKED) {
				cam->SetColor();
				cam->CameraParam.Color = rb_color->Get();
			}
			return (INT_PTR)true;
		case IDC_RADIO_MONO:
			if (wmEvent == BN_CLICKED) {
				cam->SetMono8();
				cam->CameraParam.Color = rb_color->Get();
			}
			return (INT_PTR)true;
		case IDOK:
			//cam->CamParam.Color = rb_color->Get();
			//SetCamParam(cam->CamParam);
			GetCamParam(cam->CameraParam);

			EndCtrl(hDlg);
			EndDialog(hDlg, wmId);
			return (INT_PTR)true;
		case IDCANCEL:
			SetCamParam(BackUpCamParam);
			GetCamParam(cam->CameraParam);
			EndCtrl(hDlg);
			EndDialog(hDlg, wmId);
			return (INT_PTR)true;
		}
		return (INT_PTR)false;
	}
	return false;
}

void MRC::Parameters::InitCtrl(HWND hDlg)
{
	RECT rc;
	std::wstring ws;

	cam->Stop();
	GetCamParam(cam->CameraParam);
	GetCamParam(BackUpCamParam);

	//cam->GetPixelMappingMode(ws);
	/* SetMono8sPixelMappingModeShiftleft3BitsɕύX*/
	//cam->SetMono8();
	//cam->SetPixelMappingMode(ws);

	cam->Start();
	GetWindowRect(hDlg, &rc);
	rc.right = rc.left + (LONG)cam->MAX_WIDTH;
	rc.bottom = rc.top + (LONG)cam->MAX_HEIGHT + BMP_OFFSETY;
	AdjustWindowRect(&rc, WS_OVERLAPPEDWINDOW, false);
	if (rc.top < 0) {
		rc.bottom = rc.bottom - rc.top;
		rc.top = 0;
	}
	MoveWindow(hDlg, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, true);
	GetClientRect(hDlg, &rc);
	btn_OK.reset(new DlgButton(hDlg, IDOK));

	double w_ratio, h_ratio;
	int bmp_width, bmp_height;
	w_ratio = (double)cam->MAX_WIDTH / (double)cam->CameraParam.w;
	h_ratio = (double)cam->MAX_HEIGHT / (double)cam->CameraParam.h;
	if (w_ratio <= h_ratio) {
		bmp_width = (int)cam->MAX_WIDTH;
		bmp_height = (int)((double)cam->CameraParam.h * w_ratio);
	}
	else {
		bmp_width = (int)((double)cam->CameraParam.w * h_ratio);
		bmp_height = (int)cam->MAX_HEIGHT;
	}


	bmp.reset(new DlgBitmap(hDlg, IDC_CAM_BITMAP, bmp_width, bmp_height));
	bmp->fill(WHITENESS);
	bmp->move(BMP_OFFSETX, BMP_OFFSETY);


	lb_fps.reset(new DlgLabel(hDlg, IDC_STATIC_FPS));
	//ws = (boost::wformat(L"0 fps")).str();
	//lb_fps->SetText(ws);

	sl_r.reset(new DlgSlider(hDlg, IDC_SLIDER_R, 0, 255, cam->GetGainR(), 10, 1));
	sl_g.reset(new DlgSlider(hDlg, IDC_SLIDER_G, 0, 255, cam->GetGainG(), 10, 1));
	sl_b.reset(new DlgSlider(hDlg, IDC_SLIDER_B, 0, 255, cam->GetGainB(), 10, 1));
	sl_globalgain.reset(new DlgSlider(hDlg, IDC_SLIDER_GLOBALGAIN, 0, 3, cam->GetGlobalGain(), 1, 1));
	sl_exposure.reset(new DlgSlider(hDlg, IDC_SLIDER_EXPOSURE, 0, 20000, (int)cam->GetExposureTime(), 1000, 100));

	sl_fps.reset(new DlgSlider(hDlg, IDC_SLIDER_FPS, 0, 2000, (int)cam->GetFPS(), 100, 10));

	ckb_histogram.reset(new DlgCheckbox(hDlg, IDC_CHECK_HISTOGRAM));
	ckb_histogram->Set(true);
	ckb_auto.reset(new DlgCheckbox(hDlg, IDC_CHECK_AUTO));

	ckb_auto->Set(cam->GetAutoControl());

	rb_color.reset(new DlgRadioButton(hDlg, IDC_RADIO_COLOR));
	rb_mono.reset(new DlgRadioButton(hDlg, IDC_RADIO_MONO));

	if (cam->CameraParam.Color) {
		rb_color->Set(true);
	}
	else {
		rb_mono->Set(true);
	}

	cl_bits.reset(new DlgComboList(hDlg, IDC_COMBO_BITS));

	BOOST_FOREACH(auto modeString, cam->PixelMappingModeString) {
		cl_bits->add(modeString);
	}
	cam->GetPixelMappingMode(ws);
	cl_bits->Set(ws);

	SetTimer(hDlg, ID_SETTINGCAMERA_TIMER, SETTINGCAMERA_INTERVAL, NULL);
}

INT_PTR CALLBACK MRC::Parameters::CallbackProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	Parameters* thisPtr = NULL;

	switch (message) {
	case WM_INITDIALOG:
		thisPtr = (Parameters*)lParam;
		SetWindowLongPtr(hDlg, DWLP_USER, (LONG_PTR)lParam);
		break;
	case WM_DESTROY:
		thisPtr = (Parameters*)GetWindowLongPtr(hDlg, DWLP_USER);
		SetWindowLongPtr(hDlg, DWLP_USER, (LONG_PTR)0);
		break;
	default:
		thisPtr = (Parameters*)GetWindowLongPtr(hDlg, DWLP_USER);
		break;
	}
	if (thisPtr) {
		return thisPtr->DlgProc(hDlg, message, wParam, lParam);
	}
	return false;
}

bool MRC::GetCameraParameter(MRC_CAMERA_PARAM& p)
{
	p.R = GetGainR();
	p.G = GetGainG();
	p.B = GetGainB();

	p.AutoControl = GetAutoControl();
	p.GlobalGain = GetGlobalGain();
	p.ExposureTime = GetExposureTime();

	p.ExposureTimeRaw = GetExposureTimeRaw();
	p.FPS = GetFPS();
	p.FT_TIMER = GetFT_TIMER();
	p.RowFPN = GetRowFPN();
	if (!GetPixelMappingMode(p.mode)) {
		SetMono8();
		GetPixelMappingMode(p.mode);
	}
	p.Color = IsColor();
	GetSize(p.w, p.h);
	GetOffset(p.OffsetX, p.OffsetY);
	if (camera != 0) {
		p.SerialNumber = GetMacAddress();
		p.model = GetModel();
	}

	return true;
}

bool MRC::SetCameraParameter(MRC_CAMERA_PARAM& p)
{
	//SetGainR(p.R);
	//SetGainG(p.G);
	//SetGainB(p.B);
	//SetAutoControl(p.AutoControl);


	SetGlobalGain(p.GlobalGain);


	SetRowFPN(TRUE);

	/* SetMono8sPixelMappingModeShiftleft3BitsɕύX*/
	SetMono8();
	SetPixelMappingMode(p.mode);

	SetSize(p.w, p.h);
	SetOffset(p.OffsetX, p.OffsetY);


	SetExposureTimeRaw(p.ExposureTimeRaw);
	SetExposureTime(p.ExposureTime);
	SetFT_TIMER(p.FT_TIMER);
	SetFPS(p.FPS);
	
	return true;
}

bool MRC::Save()
{
	MRC_CAMERA_PARAM p;
	GetCameraParameter(p);

	boost::filesystem::wofstream file("setting_mrc.xml");
	boost::archive::xml_woarchive oa(file);
	oa << make_nvp("MRC_Camera_Parameter", (const MRC_CAMERA_PARAM&)p);
	return true;
}

std::wstring MRC::GetMacAddress()
{
	std::wstringstream wss;
	int i;
	i = camera-1;
	wss << boost::wformat(TEXT("%x:%x:%x:%x:%x:%x"))
		% (unsigned char)(dis.param[i].Mac[0])
		% (unsigned char)(dis.param[i].Mac[1])
		% (unsigned char)(dis.param[i].Mac[2])
		% (unsigned char)(dis.param[i].Mac[3])
		% (unsigned char)(dis.param[i].Mac[4])
		% (unsigned char)(dis.param[i].Mac[5]);
	
	return wss.str();
}

std::wstring MRC::GetModel()
{
	std::wstringstream wss;
	int i;
	i = camera - 1;
	wss << boost::wformat(TEXT("%s"))
		% (char *)(dis.param[i].model);

	return wss.str();
}

int MRC::MaxWidth()
{

	return (int)MAX_WIDTH;
}

int MRC::MaxHeight()
{

	return (int)MAX_HEIGHT;
}

bool MRC::CaptureImage(CameraImage& ci, std::wstring wstr)
{
	int wh;
	IMAGE_HEADER img_header;
	WORD error;
	int img_size= (int)GetImageSize();
	static INT64 up_counter=0;
	static INT64 previous_co = 0;
	static INT64 old_exposure_time=1800;
	std::wstringstream wss;

	wh = (int)CameraParam.h * (int)CameraParam.w;
	
	if ((ci.w != (int)CameraParam.w) || (ci.h != (int)CameraParam.h) ){
		ci.w =(int)CameraParam.w;
		ci.h = (int)CameraParam.h;
		if (ci.img != NULL) delete [] ci.img;
		ci.img = new unsigned char[static_cast<size_t>(img_size)];
	}

	if ((darknoise_w != (int)CameraParam.w) || (darknoise_h != (int)CameraParam.h)) {
		darknoise_w = (int)CameraParam.w;
		darknoise_h = (int)CameraParam.h;
		if (darknoise_img != NULL) delete[] darknoise_img;
		darknoise_img = new unsigned char[static_cast<size_t>(img_size)];
		darknoise_init_counter = 0;
		if (current_img != NULL) delete[] current_img;
		current_img = new unsigned char[static_cast<size_t>(img_size)];
	}

	error = GEVGetImageBuffer(camera, &img_header, current_img);
	if (darknoise_init_counter > 10) {
		darknoise_bw_subtract_SSE2(ci.img, current_img, darknoise_img, ci.w, ci.h, wh);
		//memcpy(ci.img, current_img, wh);
	}
	if (darknoise_init_counter == 10) {
		memcpy(darknoise_img, current_img, wh);
		memcpy(ci.img, current_img, wh);
		
		//GEVSetFeatureInteger(camera, (char *)"ExposureTime", old_exposure_time);
		//wss << wformat(TEXT("ExposureTime %d")) % old_exposure_time << std::endl;
		//WriteInfo(wss.str());
		//CameraParam.ExposureTime = 2000;
		SetExposureTime(CameraParam.ExposureTime);
		darknoise_init_counter++;
	}
	if (darknoise_init_counter == 0) {
		GEVGetFeatureInteger(camera, (char *)"ExposureTime", &old_exposure_time);
		GEVSetFeatureInteger(camera, (char *)"ExposureTime", 0);
		
		wss << wformat(TEXT("ExposureTime %d")) % old_exposure_time << std::endl;
		WriteInfo(wss.str());
		wss << wformat(TEXT("ExposureTime %d")) % old_exposure_time << std::endl;
		WriteInfo(wss.str());
		wss << wformat(TEXT("ExposureTime %d")) % old_exposure_time << std::endl;
		WriteInfo(wss.str());
		wss << wformat(TEXT("ExposureTime %d")) % old_exposure_time << std::endl;
		WriteInfo(wss.str());
	}
	if (darknoise_init_counter < 10) {
		memcpy(ci.img, current_img, wh);
		darknoise_init_counter++;
	}

	if ((error) && (error != GEV_STATUS_GRAB_ERROR)) {
		std::wstring ws = TEXT("[ERROR] - GEVGetImage: ") + GetErrorMsg(error);
		WriteInfo(ws);
		return false;
	}
	QueryPerformanceCounter(&ci.CPU_FreqCount);
	ci.TimeStamp = img_header.TimeStamp*10*4/3;
	if (previous_co >= img_header.FrameCounter) up_counter++;
	ci.FrameID = img_header.FrameCounter+up_counter*65535;

	previous_co = img_header.FrameCounter;
	ci.pixelbits = 8;

	return true;
}