#pragma once
#include <iostream>
#include <sstream>
#include <string>
#include <sstream>
#include <vector>
#include <codecvt>
#include <locale>
#include <mutex>
#include <thread>
using namespace std;

struct MRC_COLOR_HISTOGRAM
{
	int R[256] = { 0 };
	int G[256] = { 0 };
	int B[256] = { 0 };
};

class CameraImage {
private:
public:
	unsigned char* img;
	//unsigned char* darknoise_img;
	//unsigned char* current_img;
	//int darknoise_init_counter;
	int w;
	int h;
	int pixelbits;
	INT64 TimeStamp;
	INT64 FrameID;
	INT64 CaptureCounter;
	LARGE_INTEGER CPU_FreqCount;
	CameraImage();
	~CameraImage();
};
struct MRC_CAMERA_PARAM 
{
	int R, G, B;
	bool AutoControl;
	int GlobalGain;
	bool RowFPN;
	bool UseFPN;
	INT64 ExposureTimeRaw;
	INT64 FPS;
	INT64 FT_TIMER;
	INT64 ExposureTime;
	bool Color;
	std::wstring mode;
	INT64 OffsetX;
	INT64 OffsetY;
	INT64 w;
	INT64 h;
	std::wstring model;
	std::wstring SerialNumber;
	MRC_CAMERA_PARAM() {
		R = G = B = 32;
		AutoControl = true;
		GlobalGain = 0;
		ExposureTime = 4000;
		Color = false;
		mode = std::wstring(TEXT("HighBits"));
		OffsetX = OffsetY = 200;
		w = 320;
		h = 240;

		UseFPN = true;
		FT_TIMER = 481;
		ExposureTimeRaw = 66;
		FPS = 250;
		RowFPN = true;
		//SerialNumber = std::wstring(TEXT("Hello"));
	}
private:
	friend class boost::serialization::access;
	template<class Archive>  void serialize(Archive& ar, unsigned int ver)
	{
		ar& BOOST_SERIALIZATION_NVP(R);
		ar& BOOST_SERIALIZATION_NVP(G);
		ar& BOOST_SERIALIZATION_NVP(B);
		ar& BOOST_SERIALIZATION_NVP(AutoControl);
		ar& BOOST_SERIALIZATION_NVP(GlobalGain);
		ar& BOOST_SERIALIZATION_NVP(Color);
		ar& BOOST_SERIALIZATION_NVP(ExposureTime);
		ar& BOOST_SERIALIZATION_NVP(ExposureTimeRaw);
		ar& BOOST_SERIALIZATION_NVP(FPS);
		ar& BOOST_SERIALIZATION_NVP(FT_TIMER);
		ar& BOOST_SERIALIZATION_NVP(UseFPN);
		ar& BOOST_SERIALIZATION_NVP(RowFPN);
		ar& BOOST_SERIALIZATION_NVP(mode);
		ar& BOOST_SERIALIZATION_NVP(OffsetX);
		ar& BOOST_SERIALIZATION_NVP(OffsetY);
		ar& BOOST_SERIALIZATION_NVP(w);
		ar& BOOST_SERIALIZATION_NVP(h);
		ar& BOOST_SERIALIZATION_NVP(model);
		ar& BOOST_SERIALIZATION_NVP(SerialNumber);
	}

};

class MRC {
private:
	static void OpenConsole();
	static void CloseConsole();

public:
	std::map<int, std::wstring> status2msg = {
	{GEV_STATUS_NOT_IMPLEMENTED,TEXT("STATUS_NOT_IMPLEMENTED")},
	{GEV_STATUS_INVALID_ADDRESS,TEXT("STATUS_INVALID_ADDRESS")},
	{GEV_STATUS_WRITE_PROTECT,TEXT("STATUS_WRITE_PROTECT")},
	{GEV_STATUS_BAD_ALIGNMENT,TEXT("STATUS_BAD_ALIGNMENT")},
	{GEV_STATUS_ACCESS_DENIED,TEXT("STATUS_ACCESS_DENIED")},
	{GEV_STATUS_BUSY,TEXT("STATUS_BUSY")},
	{GEV_STATUS_LOCAL_PROBLEM,TEXT("STATUS_LOCAL_PROBLEM")},
	{GEV_STATUS_MSG_MISMATCH,TEXT("STATUS_MSG_MISMATCH")},
	{GEV_STATUS_INVALID_PROTOCOL,TEXT("STATUS_INVALID_PROTOCOL")},
	{GEV_STATUS_NO_MSG,TEXT("STATUS_NO_MSG")},
	{GEV_STATUS_PACKET_UNAVAILABLE,TEXT("STATUS_PACKET_UNAVAILABLE")},
	{GEV_STATUS_DATA_OVERRUN,TEXT("STATUS_DATA_OVERRUN")},
	{GEV_STATUS_INVALID_HEADER,TEXT("STATUS_INVALID_HEADER")},
	{GEV_STATUS_WRONG_CONFIG,TEXT("STATUS_WRONG_CONFIG")},
	{GEV_STATUS_PACKET_NOT_YET_AVAILABLE,TEXT("STATUS_PACKET_NOT_YET_AVAILABLE")},
	{GEV_STATUS_PACKET_AND_PREV_REMOVED_FROM_MEMORY,TEXT("STATUS_PACKET_AND_PREV_REMOVED_FROM_MEMORY")},
	{GEV_STATUS_PACKET_REMOVED_FROM_MEMORY,TEXT("STATUS_PACKET_REMOVED_FROM_MEMORY")},
	{GEV_STATUS_ERROR,TEXT("STATUS_ERROR")},
	{GEV_STATUS_CAMERA_NOT_INIT,TEXT("STATUS_CAMERA_NOT_INIT")},
	{GEV_STATUS_CAMERA_ALWAYS_INIT,TEXT("STATUS_CAMERA_ALWAYS_INIT")},
	{GEV_STATUS_CANNOT_CREATE_SOCKET,TEXT("STATUS_CANNOT_CREATE_SOCKET")},
	{GEV_STATUS_SEND_ERROR,TEXT("STATUS_SEND_ERROR")},
	{GEV_STATUS_RECEIVE_ERROR,TEXT("STATUS_RECEIVE_ERROR")},
	{GEV_STATUS_CANNOT_ALLOC_MEMORY,TEXT("STATUS_CANNOT_ALLOC_MEMORY")},
	{GEV_STATUS_TIMEOUT,TEXT("STATUS_TIMEOUT")},
	{GEV_STATUS_SOCKET_ERROR,TEXT("STATUS_SOCKET_ERROR")},
	{GEV_STATUS_INVALID_ACK,TEXT("STATUS_INVALID_ACK")},
	{GEV_STATUS_CANNOT_START_THREAD,TEXT("STATUS_CANNOT_START_THREAD")},
	{GEV_STATUS_CANNOT_SET_SOCKET_OPT,TEXT("STATUS_CANNOT_SET_SOCKET_OPT")},
	{GEV_STATUS_CANNOT_OPEN_DRIVER,TEXT("STATUS_CANNOT_OPEN_DRIVER")},
	{GEV_STATUS_HEARTBEAT_READ_ERROR,TEXT("STATUS_HEARTBEAT_READ_ERROR")},
	{GEV_STATUS_EVALUATION_EXPIRED,TEXT("STATUS_EVALUATION_EXPIRED")},
	{GEV_STATUS_GRAB_ERROR,TEXT("STATUS_GRAB_ERROR")},
	{GEV_STATUS_XML_READ_ERROR,TEXT("STATUS_XML_READ_ERROR")},
	{GEV_STATUS_XML_OPEN_ERROR,TEXT("STATUS_XML_OPEN_ERROR")},
	{GEV_STATUS_XML_FEATURE_ERROR,TEXT("STATUS_XML_FEATURE_ERROR")},
	{GEV_STATUS_XML_COMMAND_ERROR,TEXT("STATUS_XML_COMMAND_ERROR")},
	{GEV_STATUS_GAIN_NOT_SUPPORTED,TEXT("STATUS_GAIN_NOT_SUPPORTED")},
	{GEV_STATUS_EXPOSURE_NOT_SUPPORTED,TEXT("STATUS_EXPOSURE_NOT_SUPPORTED")},
	{GEV_STATUS_CANNOT_GET_ADAPTER_INFO,TEXT("STATUS_CANNOT_GET_ADAPTER_INFO")},
	{GEV_STATUS_ERROR_INVALID_HANDLE,TEXT("STATUS_ERROR_INVALID_HANDLE")},
	{GEV_STATUS_CLINK_SET_BAUD,TEXT("STATUS_CLINK_SET_BAUD")},
	{GEV_STATUS_CLINK_SEND_BUFFER_FULL,TEXT("STATUS_CLINK_SEND_BUFFER_FULL")},
	{GEV_STATUS_CLINK_RECEIVE_BUFFER_NO_DATA,TEXT("STATUS_CLINK_RECEIVE_BUFFER_NO_DATA")},
	{GEV_STATUS_FEATURE_NOT_AVAILABLE,TEXT("STATUS_FEATURE_NOT_AVAILABLE")},
	{GEV_STATUS_MATH_PARSER_ERROR,TEXT("STATUS_MATH_PARSER_ERROR")},
	{GEV_STATUS_FEATURE_ITEM_NOT_AVAILABLE,TEXT("STATUS_FEATURE_ITEM_NOT_AVAILABLE")},
	{GEV_STATUS_NOT_SUPPORTED,TEXT("STATUS_NOT_SUPPORTED")},
	{GEV_STATUS_GET_URL_ERROR,TEXT("STATUS_GET_URL_ERROR")},
	{GEV_STATUS_READ_XML_MEM_ERROR,TEXT("STATUS_READ_XML_MEM_ERROR")},
	{GEV_STATUS_XML_SIZE_ERROR,TEXT("STATUS_XML_SIZE_ERROR")},
	{GEV_STATUS_XML_ZIP_ERROR,TEXT("STATUS_XML_ZIP_ERROR")},
	{GEV_STATUS_XML_ROOT_ERROR,TEXT("STATUS_XML_ROOT_ERROR")},
	{GEV_STATUS_XML_FILE_ERROR,TEXT("STATUS_XML_FILE_ERROR")},
	{GEV_STATUS_DIFFERENT_IMAGE_HEADER,TEXT("STATUS_DIFFERENT_IMAGE_HEADER")},
	{GEV_STATUS_XML_SCHEMA_ERROR,TEXT("STATUS_XML_SCHEMA_ERROR")},
	{GEV_STATUS_XML_STYLESHEET_ERROR,TEXT("STATUS_XML_STYLESHEET_ERROR")},
	{GEV_STATUS_FEATURE_LIST_ERROR,TEXT("STATUS_FEATURE_LIST_ERROR")},
	{GEV_STATUS_ALREADY_OPEN,TEXT("STATUS_ALREADY_OPEN")},
	{GEV_STATUS_TEST_PACKET_DATA_ERROR,TEXT("STATUS_TEST_PACKET_DATA_ERROR")},
	{GEV_STATUS_FEATURE_NOT_FLOAT,TEXT("STATUS_FEATURE_NOT_FLOAT")},
	{GEV_STATUS_XML_DLL_NOT_FOUND,TEXT("STATUS_XML_DLL_NOT_FOUND")},
	{GEV_STATUS_XML_NOT_INIT,TEXT("STATUS_XML_NOT_INIT")}
	};
	static HANDLE hStdOutput;

	BYTE camera;
	DISCOVERY dis;
	bool IsStream = false;
	INT64 MAX_WIDTH = 1280;
	INT64 MAX_HEIGHT = 960;
	INT64 MIN_WIDTH = 4;
	INT64 MIN_HEIGHT = 16;
	INT64 INC_WIDTH = 4;
	INT64 INC_HEIGHT = 2;

	INT64 MAX_OFFSETX = 1279;
	INT64 MIN_OFFSETX = 0;
	INT64 MAX_OFFSETY = 959;
	INT64 MIN_OFFSETY = 0;
	INT64 INC_OFFSETX = 2;
	INT64 INC_OFFSETY = 2;
	int darknoise_init_counter = 0;
	int darknoise_w;
	int darknoise_h;
	unsigned char* darknoise_img;
	unsigned char* current_img;
	double fps;
	MRC_CAMERA_PARAM CameraParam;
	MRC(void);
	~MRC(void);

	bool OpenStream();
	bool CloseStream();
	std::wstring GetErrorMsg(WORD error);
	bool IsReady();
	bool IsStreaming(void);
	void Start(void);
	void Stop(void);
	void SetColor(void);
	void SetMono8(void);
	bool IsColor(void);
    int a2t(char* src, TCHAR* des, rsize_t num);
	int t2a(TCHAR* src, char* des, size_t num);

	std::string wstr2str(const std::wstring& wstr);
	std::wstring str2wstr(const std::string& str);


	//
	// mode -> "LowBits","Shiftleft1Bit","MiddleBits","Shiftleft3Bits","HighBits"
	//std::vector <std::wstring> PixelMappingModeString = { TEXT("HighBits"),TEXT("Shiftleft3Bits") ,TEXT("MiddleBits") ,TEXT("Shiftleft1Bit") ,TEXT("LowBits") };
	std::vector <std::wstring> PixelMappingModeString = { TEXT("HighBits") ,TEXT("MiddleBits")  ,TEXT("LowBits") };
	bool SetPixelMappingMode(char* mode);
	bool SetPixelMappingMode(std::wstring ws);
	bool GetPixelMappingMode(std::wstring &ws);
	//
	// GlobalGain mode -> 0,1,2,3;
	//
	bool SetGlobalGain(int mode);
	int GetGlobalGain(void);

	bool SetExposureTime(INT64 micro_second);
	INT64 GetExposureTime(void);

	bool SetAutoControl(bool mode);
	bool GetAutoControl(void);

	bool GetMaxMinInc(char* feature_name, INT64& max, INT64& min, INT64& inc);

	bool SetSize(INT64 width, INT64 height);
	void GetSize(INT64& width, INT64& height);
	void GetMaxSize(INT64& width, INT64& height);
	bool SetOffset(INT64 x, INT64 y);
	void GetOffset(INT64& x, INT64& y);

	
	void GetHistogram(CameraImage& ci, MRC_COLOR_HISTOGRAM& hist);
	INT64 GetImageSize(void);

	bool SetGainR(int red);
	int GetGainR(void);

	bool SetGainG(int green);
	int GetGainG(void);

	bool SetGainB(int blue);
	int GetGainB(void);

	bool SetFPS(int64 fps);
	int GetFPS(void);

	bool SetRowFPN(int fpn);
	bool GetRowFPN(void);

	bool SetExposureTimeRaw(INT64);
	INT64 GetExposureTimeRaw(void);

	bool SetFT_TIMER(INT64);
	INT64 GetFT_TIMER(void);


	bool TempProc(void);
	static void WriteInfo(std::wstring ws);
	static BYTE WINAPI error_callback_func(BYTE cam_nr, char* error_str);
	static BYTE WINAPI msg_callback_func(BYTE cam_nr, MESSAGECHANNEL_PARAMETER mcparam);

	bool DialogSelect(HWND hWnd =NULL);
	bool DialogRegion(HWND hWnd = NULL);
	bool DialogParameters(HWND hWnd = NULL);

	bool Save();
	bool GetCameraParameter(MRC_CAMERA_PARAM& p);
	bool SetCameraParameter(MRC_CAMERA_PARAM& p);
	bool CaptureImage(CameraImage& ci, std::wstring wstr);

	std::wstring GetMacAddress();
	std::wstring GetModel();

	int MaxWidth();
	int MaxHeight();
	class DlgSelect {
	public:
		HWND hWndParent;

		MRC* CamPtr;
		std::unique_ptr <DlgListBox> lb;
		DlgSelect();
		~DlgSelect();

		INT_PTR Open(HWND hWndParent, MRC& mrc);
		bool InitCtrl(HWND hDlg);
		void EndCtrl(HWND hDlg);
		bool DlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
		static INT_PTR CALLBACK CallbackProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	};

	class InputRegion
	{
	private:
	public:
		MRC* cam;
		HWND hWndParent;
	

		POINTS start, end;

		HCURSOR hCursorArrow, hCursorSIZENWSE, hCursorSIZENESW;

		//BYTE* img = NULL;
		CameraImage ci;
		MRC_CAMERA_PARAM BackUpCamParam;
		std::unique_ptr<DlgBitmap> bmp;
		std::unique_ptr<DlgButton> btn_OK;
		std::unique_ptr<DlgSlider> sl_r;
		std::unique_ptr<DlgSlider> sl_g;
		std::unique_ptr<DlgSlider> sl_b;
		std::unique_ptr<DlgSlider> sl_globalgain;
		std::unique_ptr<DlgSlider> sl_exposure;
		std::unique_ptr<DlgCheckbox> ckb_histogram;
		std::unique_ptr<DlgCheckbox> ckb_auto;
		std::unique_ptr<DlgRadioButton> rb_color;
		std::unique_ptr<DlgRadioButton> rb_mono;
		std::unique_ptr<DlgComboList> cl_bits;

		std::unique_ptr<DlgEditBox> eb_width;
		std::unique_ptr<DlgEditBox> eb_height;
		std::unique_ptr<DlgEditBox> eb_offsetx;
		std::unique_ptr<DlgEditBox> eb_offsety;

		std::unique_ptr<DlgSpin> sp_width;
		std::unique_ptr<DlgSpin> sp_height;
		std::unique_ptr<DlgSpin> sp_offsetx;
		std::unique_ptr<DlgSpin> sp_offsety;

		void SetInputOffset();
		void SetInputSize();
		void SP_SetOffsetX(void);
		void SP_SetOffsetY(void);
		void SP_SetWidth(void);
		void SP_SetHeight(void);



		InputRegion();
		~InputRegion();
		void InitCtrl(HWND hDlg);
		void EndCtrl(HWND hDlg);
		INT_PTR Open(HWND hWnd, MRC& cam);
		BOOL DlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lPalram);
		static 	INT_PTR CALLBACK CallbackProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	};

	class Parameters
	{
	private:
	public:
		MRC* cam;
		HWND hWndParent;

		CameraImage ci;
		MRC_CAMERA_PARAM BackUpCamParam;
		std::unique_ptr<DlgBitmap> bmp;
		std::unique_ptr<DlgButton> btn_OK;
		std::unique_ptr<DlgLabel> lb_fps;
		std::unique_ptr<DlgSlider> sl_r;
		std::unique_ptr<DlgSlider> sl_g;
		std::unique_ptr<DlgSlider> sl_b;
		std::unique_ptr<DlgSlider> sl_globalgain;
		std::unique_ptr<DlgSlider> sl_exposure;
		std::unique_ptr<DlgSlider> sl_fps;
		std::unique_ptr<DlgCheckbox> ckb_histogram;
		std::unique_ptr<DlgCheckbox> ckb_auto;
		std::unique_ptr<DlgRadioButton> rb_color;
		std::unique_ptr<DlgRadioButton> rb_mono;
		std::unique_ptr<DlgComboList> cl_bits;

		void GetCamParam(MRC_CAMERA_PARAM& p);
		void SetCamParam(MRC_CAMERA_PARAM& p);

		Parameters();
		~Parameters();
		void InitCtrl(HWND hDlg);
		void EndCtrl(HWND hDlg);
		INT_PTR Open(HWND hWnd, MRC& cam);
		BOOL DlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lPalram);
		static 	INT_PTR CALLBACK CallbackProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	};
};

